package com.emato.cus.supervise.biz.acqGoodsMoveSchedule.up;

import com.emato.cus.supervise.biz.DataConversion;
import com.emato.cus.supervise.biz.EmsClassConversion;
import com.emato.cus.supervise.constant.Dict;
import com.emato.cus.supervise.domain.WmsAcqGoodsMove;
import com.emato.cus.supervise.domainCus.CusAcqGoodsMove05;
import com.emato.cus.supervise.domainOms.OmsMerchInfo;
import com.emato.cus.supervise.domainOms.OmsProductInfo;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author zengjunlin
 * @version 1.0
 * 2018-02-03 15:40
 */
@Component
public class AcqGoodsMoveUpDataConversion implements DataConversion {

    private final  static Logger logger = LoggerFactory.getLogger(AcqGoodsMoveUpDataConversion.class);

    /**
     * 货物库位移动数据
     *
     * @param params
     * @return
     */
    @Override
    public List conversion(Map params) {
        logger.info("货物库位移动数据开始转换oms，wms 组装cus数据===========================");
        List<WmsAcqGoodsMove> wmsList = (List<WmsAcqGoodsMove>) params.get("wmsList");

        List<OmsMerchInfo> omsList = (List<OmsMerchInfo>) params.get("omsList");
        final Map<String, OmsMerchInfo> omsMerchInfoMap = omsList.stream().collect(Collectors.toMap(k -> k.getMerchSn() + "_" + k.getEmsClassType(), v -> v));

        List<OmsProductInfo> productInfoList = (List<OmsProductInfo>) params.get("productInfoList");
        final Map<String,OmsProductInfo> omsProductInfoMap = new HashMap<>();
        //final Map<String,OmsProductInfo> omsProductInfoMap = productInfoList.stream().collect(Collectors.toMap(k->k.getSkuNo(), v->v));
        for (int i = 0, len = productInfoList.size(); i < len; i++) {
            if (omsProductInfoMap.containsKey(productInfoList.get(i).getSkuNo())) {
                continue;
            } else {
                omsProductInfoMap.put(productInfoList.get(i).getSkuNo(), productInfoList.get(i));
            }
        }


        List<CusAcqGoodsMove05> cusAcqGoodsMove05List = Lists.newArrayList();

        for (int i = 0; i < wmsList.size(); i++) {
            // #20190521001
            // 未作try catch 导致一个sku数据异常，全部终止
            try {
                WmsAcqGoodsMove wmsAcqGoodsMove = wmsList.get(i);

                String seatNo = null;
                if (wmsAcqGoodsMove.getTargetSeatNo() == null) {
                    logger.debug("sku：{}, 目标库位为空", wmsAcqGoodsMove.getSkuNo());
                    if(wmsAcqGoodsMove.getOriginSeatNo() == null) {
                        logger.debug("sku：{}, 源库位为空", wmsAcqGoodsMove.getSkuNo());
                        continue;
                    } else {
                        seatNo = wmsAcqGoodsMove.getOriginSeatNo();
                    }
                } else {
                    seatNo = wmsAcqGoodsMove.getTargetSeatNo();
                }


                // 库位向账册分类转换
                String emsClassType = EmsClassConversion.convertBySeatNo(seatNo);
                // 拼接商户账册分类
                String merchEmsType = wmsAcqGoodsMove.getMerchSn() + "_" + emsClassType;
                if (!omsMerchInfoMap.containsKey(merchEmsType)) {
                    logger.error("当前转换的商户账册分类为: {}, WMS传回的商户在OMS中无此账册分类", merchEmsType);
                    continue;
                }
                OmsMerchInfo omsMerchInfo = omsMerchInfoMap.get(merchEmsType);


                OmsProductInfo omsProductInfo = omsProductInfoMap.get(wmsAcqGoodsMove.getSkuNo());
                String osku = omsProductInfo.getSkuNo();
                if (StringUtils.isBlank(osku)) {
                    logger.error("当前转换的sku为: {}, 对应的产品备案数据异常", osku);
                    continue;
                }
                //logger.debug("当前转换的sku为: {}", osku);

                String targetSeatNo = wmsAcqGoodsMove.getTargetSeatNo();


                CusAcqGoodsMove05 cusAcqGoodsMove05 = new CusAcqGoodsMove05();
                cusAcqGoodsMove05.setId(wmsAcqGoodsMove.getId());
                if("上架".equals(wmsAcqGoodsMove.getTaskType())){
                    cusAcqGoodsMove05.setShelfLoadTime(wmsAcqGoodsMove.getCreated());
                }
                if("批量拣选".equals(wmsAcqGoodsMove.getTaskType())){
                    cusAcqGoodsMove05.setShelfUnloadTime(wmsAcqGoodsMove.getCreated());
                }
                cusAcqGoodsMove05.setTargetSeatNo(wmsAcqGoodsMove.getTargetSeatNo());
                cusAcqGoodsMove05.setOriginSeatNo(wmsAcqGoodsMove.getOriginSeatNo());
                cusAcqGoodsMove05.setGoodsName(wmsAcqGoodsMove.getGoodsName());

                cusAcqGoodsMove05.setSkuNo(wmsAcqGoodsMove.getSkuNo());
                cusAcqGoodsMove05.setGoodsId(wmsAcqGoodsMove.getGoodsId());
                cusAcqGoodsMove05.setCodeTs(omsProductInfo.getCodeTs());
                cusAcqGoodsMove05.setgUnit(wmsAcqGoodsMove.getgUnit());
                cusAcqGoodsMove05.setgQty(wmsAcqGoodsMove.getgQty());

                cusAcqGoodsMove05.setLegalQty(omsProductInfo.getLegalQty());
                cusAcqGoodsMove05.setLegalUnit(omsProductInfo.getLegalUnit());
                cusAcqGoodsMove05.setGoodsModel(omsProductInfo.getGoodsModel());

                cusAcqGoodsMove05.setLocalEmsNo(omsMerchInfo.getCusEmsNo());
                cusAcqGoodsMove05.setStoreCompanyName(Dict.storeCompanyName);
                cusAcqGoodsMove05.setStoreUscCode(Dict.storeUscCode);
                cusAcqGoodsMove05.setStoreCustomsCode(Dict.storeCustomsCode);
                if (Dict.storeCodeMap.containsKey(wmsAcqGoodsMove.getStoreCode())) {
                    cusAcqGoodsMove05.setStoreCode(Dict.storeCodeMap.get(wmsAcqGoodsMove.getStoreCode()));
                } else {
                    cusAcqGoodsMove05.setStoreCode(Dict.storeCodeMap.get("Others"));
                }
                cusAcqGoodsMove05.setStoreCode(Dict.storeCode);
                cusAcqGoodsMove05.setCreateTime(LocalDateTime.now());
                cusAcqGoodsMove05.setCreaterSn(Dict.operater);
                cusAcqGoodsMove05.setModerSn(Dict.operater);
                cusAcqGoodsMove05.setModTime(LocalDateTime.now());

                cusAcqGoodsMove05List.add(cusAcqGoodsMove05);
            }catch (Exception e){
                logger.error("货物库位移动组装cus数据系统异常", e);
            }finally {
                continue;
            }
        }
        logger.info("货物库位移动转换oms，wms 组装cus数据结束===========================");
        return cusAcqGoodsMove05List;
    }
}
